-- Name: BSU_bunkerSiloOverwrites
-- Author: Chissel

BSU_bunkerSiloOverwrites = {}


function BSU_bunkerSiloOverwrites:update(dt)
    if self:getCanInteract(true) then
        local fillType = g_fillTypeManager:getFillTypeByIndex(self.fermentingFillType)
        if fillType ~= nil then
            g_currentMission:addExtraPrintText(string.format("%s: %s", g_i18n:getText("info_BSU_SILO_COVER"), fillType.title))
        end
    end
end

function BSU_bunkerSiloOverwrites.setState(self, superFunc, state, showNotification)
    if state ~= self.state then
        if state == BunkerSilo.STATE_FILL then
            if self.state == BunkerSilo.STATE_CLOSED then
                self.compactedFillLevel = self.fillLevel
                self.compactedPercent = 100

                self:switchFillTypeToInputFillType()
            else
                self.compactedFillLevel = 0
                self.compactedPercent = 0
            end

            self.fermentingPercent = 0
            self.isOpenedAtFront = false
            self.isOpenedAtBack = false
            self.bunkerSiloArea.offsetFront = 0
            self.bunkerSiloArea.offsetBack = 0

            if showNotification then
                self:showBunkerMessage(g_i18n:getText("ingameNotification_bunkerSiloIsEmpty"))
            end

            if self.isServer then
                g_densityMapHeightManager:removeFixedFillTypesArea(self.bunkerSiloArea)
                g_densityMapHeightManager:setConvertingFillTypeAreas(self.bunkerSiloArea, self.acceptedFillTypes, self.inputFillType)
            end

        elseif state == BunkerSilo.STATE_CLOSED then

            if self.isServer then
                -- change fillType
                local area = self.bunkerSiloArea
                local offsetFront = self:getBunkerAreaOffset(true, 0, self.inputFillType)
                local offsetBack = self:getBunkerAreaOffset(false, 0, self.inputFillType)

                local x0 = area.sx + (offsetFront * area.dhx_norm)
                local z0 = area.sz + (offsetFront * area.dhz_norm)
                local x1 = x0 + area.dwx
                local z1 = z0 + area.dwz
                local x2 = area.sx + area.dhx - (offsetBack * area.dhx_norm)
                local z2 = area.sz + area.dhz - (offsetBack * area.dhz_norm)

                local changed = DensityMapHeightUtil.changeFillTypeAtArea(x0, z0, x1, z1, x2, z2, self.inputFillType, self.fermentingFillType)

                g_densityMapHeightManager:removeFixedFillTypesArea(self.bunkerSiloArea)
                g_densityMapHeightManager:removeConvertingFillTypeAreas(self.bunkerSiloArea)
            end

            if showNotification then
                self:showBunkerMessage(g_i18n:getText("ingameNotification_bunkerSiloCovered"))
            end

        elseif state == BunkerSilo.STATE_FERMENTED then
            if showNotification then
                self:showBunkerMessage(g_i18n:getText("ingameNotification_bunkerSiloDoneFermenting"))
            end

        elseif state == BunkerSilo.STATE_DRAIN then

            self.bunkerSiloArea.offsetFront = 0
            self.bunkerSiloArea.offsetBack = 0

            if showNotification then
                self:showBunkerMessage(g_i18n:getText("ingameNotification_bunkerSiloOpened"))
            end

            if self.isServer then
                g_densityMapHeightManager:removeConvertingFillTypeAreas(self.bunkerSiloArea)
                local fillTypes = {}
                fillTypes[self.outputFillType] = true
                g_densityMapHeightManager:setFixedFillTypesArea(self.bunkerSiloArea, fillTypes)
            end
        end

        self.state = state
        if self.isServer then
            self:raiseDirtyFlags(self.bunkerSiloDirtyFlag)
        end
    end
end

-- Close cover

function BunkerSilo:closeCoverCallback()
    local ix,iy,iz = self:getInteractionPosition()
    local closeAtFront = self:getIsCloserToFront(ix,iy,iz)

    self:closeCover(closeAtFront)

    if not self.isServer then
        g_client:getServerConnection():sendEvent(BSU_closeCoverEvent.new(self, closeAtFront))
    end
end

function BunkerSilo:closeCover(closeAtFront)
    if closeAtFront and self.isOpenedAtFront then
        self.isOpenedAtFront = false
    elseif not closeAtFront and self.isOpenedAtBack then
        self.isOpenedAtBack = false
    end

    if not self.isOpenedAtFront and not self.isOpenedAtBack then
        self:setState(BunkerSilo.STATE_FERMENTED)

        if self.isServer then
            g_densityMapHeightManager:removeFixedFillTypesArea(self.bunkerSiloArea)
            g_densityMapHeightManager:removeConvertingFillTypeAreas(self.bunkerSiloArea)

            local offsetFront = self:getBunkerAreaOffset(true, 0, self.inputFillType)
            local offsetBack = self:getBunkerAreaOffset(false, 0, self.inputFillType)

            self:raiseDirtyFlags(self.bunkerSiloDirtyFlag)
        end
    end

    self:switchFillTypeToFermenting(closeAtFront)

    self:showBunkerMessage(g_i18n:getText("ingameNotification_bunkerSiloCovered"))
end

function BunkerSilo:switchFillTypeToFermenting(switchAtFront)
    local a0x, a0z
    local a1x, a1z
    local a2x, a2z
    local area = self.bunkerSiloArea
    local offset = 0

    local a = area.hx - area.sx
    local b = area.hz - area.sz

    local length = math.sqrt(a^2 + b^2) / 2

    if switchAtFront then
        a0x, a0z = area.sx + (offset * area.dhx_norm), area.sz + (offset * area.dhz_norm)
        a1x, a1z = a0x + area.dwx, a0z + area.dwz
        a2x, a2z = area.sx + ((offset + length) * area.dhx_norm), area.sz + ((offset + length) * area.dhz_norm)
    else
        a0x, a0z = area.hx - (offset * area.dhx_norm), area.hz - (offset * area.dhz_norm)
        a1x, a1z = a0x + area.dwx, a0z + area.dwz
        a2x, a2z = area.hx - ((offset + length) * area.dhx_norm), area.hz - ((offset + length) * area.dhz_norm)
    end

    DensityMapHeightUtil.changeFillTypeAtArea(a0x,a0z, a1x,a1z, a2x,a2z, self.outputFillType, self.fermentingFillType)
end

function BunkerSilo:switchFillTypeToInputFillType()
    local a0x, a0z
    local a1x, a1z
    local a2x, a2z
    local area = self.bunkerSiloArea
    local offset = 0

    local a = area.hx - area.sx
    local b = area.hz - area.sz

    local length = math.sqrt(a^2 + b^2)

    a0x, a0z = area.sx + (offset * area.dhx_norm), area.sz + (offset * area.dhz_norm)
    a1x, a1z = a0x + area.dwx, a0z + area.dwz
    a2x, a2z = area.sx + ((offset + length) * area.dhx_norm), area.sz + ((offset + length) * area.dhz_norm)

    DensityMapHeightUtil.changeFillTypeAtArea(a0x,a0z, a1x,a1z, a2x,a2z, self.fermentingFillType, self.inputFillType)
end

-- Reopen cover

function BunkerSilo:reopenCoverCallback()
    if self.isServer then
        self:reopenCover()
    else
        g_client:getServerConnection():sendEvent(BSU_reopenCoverEvent.new(self))
    end
end

function BunkerSilo:reopenCover()
    self:setState(BunkerSilo.STATE_FILL)

    self:showBunkerMessage(g_i18n:getText("ingameNotification_bunkerSiloOpened"))
end

-- Clear area


function BunkerSilo:openClearAreaDialog()
    g_gui:showYesNoDialog({title=g_i18n:getText("BSU_CLEAR_AREA_DIALOG_TITLE"), text=g_i18n:getText("BSU_CLEAR_AREA_DIALOG_TEXT"), callback=self.clearAreaCallback, target=self})
end

function BunkerSilo:clearAreaCallback()
    if self.isServer then
        self:clearArea()
    else
        g_client:getServerConnection():sendEvent(BSU_clearAreaEvent.new(self))
    end
end

function BunkerSilo:clearArea()
    self:setState(BunkerSilo.STATE_DRAIN)
    self.fermentingPercent = 0
    self.compactedPercent = 0
    self.bunkerSiloArea.offsetFront = 0
    self.bunkerSiloArea.offsetBack = 0
    self.isOpenedAtFront = false
    self.isOpenedAtBack = false

    self:clearSiloArea()
    self:raiseDirtyFlags(self.bunkerSiloDirtyFlag)

    self:showBunkerMessage(g_i18n:getText("ingameNotification_bunkerSiloIsEmpty"))
end

-- Change cover

local function indexOf(array, value)
    for i, v in ipairs(array) do
        if v == value then
            return i
        end
    end

    return 0
end

function BunkerSilo:changeCoverCallback()
    local availableFillTypes = BSU_bunkerSiloUtilities.availableFillTypes
    local index = indexOf(availableFillTypes, self.fermentingFillType)
    local newIndex = index + 1

    if newIndex > #availableFillTypes then
        newIndex = 1
    end

    local fillType = availableFillTypes[newIndex]

    if fillType == nil then
        print("Error BSU: FillType not available!")
        return
    end

    self:changeCover(fillType)

    if not self.isServer then
        g_client:getServerConnection():sendEvent(BSU_changeCoverEvent.new(self, fillType))
    end
end

function BunkerSilo:changeCover(fillType)
    self:switchFillTypeFromTo(self.fermentingFillType, fillType)
    self.fermentingFillType = fillType
end

function BunkerSilo:switchFillTypeFromTo(from, to)
    local a0x, a0z
    local a1x, a1z
    local a2x, a2z
    local area = self.bunkerSiloArea
    local offset = 0

    local a = area.hx - area.sx
    local b = area.hz - area.sz

    local length = math.sqrt(a^2 + b^2)

    a0x, a0z = area.sx + (offset * area.dhx_norm), area.sz + (offset * area.dhz_norm)
    a1x, a1z = a0x + area.dwx, a0z + area.dwz
    a2x, a2z = area.sx + ((offset + length) * area.dhx_norm), area.sz + ((offset + length) * area.dhz_norm)

    DensityMapHeightUtil.changeFillTypeAtArea(a0x, a0z, a1x, a1z, a2x, a2z, from, to)
end